;Footprint code for STILT particle files

FUNCTION superiorfoot,mo,dy,hr,yr=yr,btimes=btimes,lats=lats,lons=lons,xres=xres,yres=yres,flats=flats,flons=flons

  IF n_elements(yr) EQ 0 THEN yr = 2004
  IF n_elements(btimes) EQ 0 THEN btimes = [0,72] ;three day influence
  IF n_elements(lats) EQ 0 THEN lats = [41.25,49.125]
  IF n_elements(lons) EQ 0 THEN lons = [-93.25,-83.875]
  IF n_elements(xres) EQ 0 THEN xres = 0.375
  IF n_elements(yres) EQ 0 THEN yres = 0.375

  lat = numgen(lats[0],lats[1],yres)
  lon = numgen(lons[0],lons[1],xres)

  dr = mydocs()+'superior/stilt/txt/'
  fname = dr + string(yr,format='(i4.4)')+'x'+string(mo,format='(i2.2)')+'x'+string(dy,format='(i2.2)')+'x'+string(hr,format='(i2.2)')+'x45.93Nx090.27Wx00396.txt'
  print,'Reading ',fname
  ln = file_lines(fname)
  part = fltarr(13,ln-1)
  openr,fl,fname,/get_lun
  hdr = ''
  readf,fl,hdr
  readf,fl,part
  free_lun,fl
  origpart = part

;find number of particles
  pindex = part[1,*]
  npart = float(n_elements(uniq(pindex[sort(pindex)])))

;sort data by time
  psort = sort(part[0,*]*(-1))
  part = part[*,psort]

;extract particle locations and influence
  btime = part[0,*]/(-60.0)
  plat = part[2,*]
  plon = part[3,*]
  pagl = part[4,*]
  pgrnd = part[5,*]
  pfoot = part[6,*]
  pzi = part[9,*]
  pdmass = part[12,*]

print,'  Normalizing'
;normalize by mass
  nbtime = uniq(btime)
  FOR i = 0,n_elements(nbtime)-1 DO BEGIN
    gpart = where(btime EQ btime[nbtime[i]],ngpart)
    IF ngpart GT 0 THEN BEGIN
      mdmass = mean(pdmass[gpart])
      IF mdmass GT 0 THEN pdmass[gpart]/=mdmass ELSE pdmass[gpart]=0.0
    ENDIF 
  ENDFOR 

;create grid
  fprint = make_array(n_elements(lon)-1,n_elements(lat)-1,n_elements(btimes)-1,/float,value=0.0)
  flats =  make_array(n_elements(lon)-1,n_elements(lat)-1,/float)
  flons = flats

;remove non surface influence and points outside domain
  vpart = where((pfoot GT 0) AND (plat GE lats[0]) AND (plat LE lats[1]) AND (plon GE lons[0]) AND (plon LE lons[1]) AND (btime GE min(btimes)) AND (btime LE max(btimes)),nvpart)
  IF nvpart GT 0 THEN BEGIN 
    btime = btime[vpart]
    plat = plat[vpart]
    plon = plon[vpart]
    pfoot = pfoot[vpart]
    pdmass = pdmass[vpart]
    pagl = pagl[vpart]
    pgrnd = pgrnd[vpart]
    pzi = pzi[vpart]
  ENDIF ELSE return,fprint

  pinf = pdmass * pfoot  ;* 60.0

print,'  Computing footprint'
;loop through grid
  FOR t = 0,n_elements(btimes)-2 DO BEGIN 
 ;   print,'  Time ',btimes[t+1]
    FOR x = 0,n_elements(lon)-2 DO BEGIN
 ;     print,'Lon ',lon[x]
      FOR y = 0,n_elements(lat)-2 DO BEGIN
        IF t EQ 1 THEN flats[x,y] = lat[y]
        IF t EQ 1 THEN flons[x,y] = lon[x]
        fpart = where((plon GE lon[x]) AND (plon LT lon[x+1]) $
                      AND (plat GE lat[y]) AND (plat LT lat[y+1]) $
                      AND (btime GT btimes[t]) AND (btime LE btimes[t+1]),nfpart)
        IF nfpart GT 0 THEN fprint[x,y,t] = total(pinf[fpart])/npart
      ENDFOR 
    ENDFOR 
  ENDFOR 

;output
  return,reform(fprint)

END

PRO superiorfoot_all,over=over,dr=dr
;run through each day and hour
;save footprint
;also save lat, lon, and water (from NARR lndmask)
  IF n_elements(dr) EQ 0 THEN dr = mydocs()+'superior/stilt/foot/'
  FOR m = 7,8 DO BEGIN
    FOR d = 1,31 DO BEGIN
      FOR h = 0,23 DO BEGIN
        fname = '2004x'+string(m,format='(i2.2)')+'x'+string(d,format='(i2.2)')+'x'+string(h,format='(i2.2)')+'xWLEF396foot.txt'
        IF (keyword_set(over)) OR (~file_test(dr+fname,/read)) THEN BEGIN 
          f = superiorfoot(m,d,h)
          openw,fl,dr+fname,/get_lun
          printf,fl,f,format='(25(g12.5," "))'
          free_lun,fl
        ENDIF ELSE print,fname,' already exists'
     ENDFOR
    ENDFOR
  ENDFOR 
END


  ; landsfc = readnarr(2008,1,1,'landsfc',/fixed,lats=latr,lons=lonr)
  ;  lats_ln = landsfc.landsfc.lat
  ;  lons_ln = landsfc.landsfc.lon
  ;  land = landsfc.landsfc.data
  ;  landmask = rebin(land,24,10)
